@@ -1,14 +1,16 @@
 import os
 from os.path import join, basename, normpath, abspath, dirname
-from os.path import split, expanduser, exists, isfile
+from os.path import split, expanduser, exists, isfile, isdir
 
 from gettext import gettext as _
 
 import gobject
-import gtk, gnome.ui
+import gtk, gnome, gnome.ui
 import deskbar, deskbar.Indexer
 import deskbar.Handler
 
+from threading import Thread
+
 HANDLERS = {
 	"FileFolderHandler" : {
 		"name": _("Files and Folders"),
@@ -24,7 +26,7 @@ class FileMatch(deskbar.Match.Match):
 		self.absname = absname
 				
 	def action(self, text=None):
-		gobject.spawn_async(["gnome-open", self.absname], flags=gobject.SPAWN_SEARCH_PATH)
+		gnome.url_show(self.absname)
 		
 	def get_category(self):
 		return "files"
@@ -43,7 +45,7 @@ class FolderMatch(deskbar.Match.Match):
 		self.absname = absname
 		
 	def action(self, text=None):
-		gobject.spawn_async(["nautilus", self.absname], flags=gobject.SPAWN_SEARCH_PATH)
+		gnome.url_show(self.absname)
 	
 	def get_category(self):
 		return "places"
@@ -57,19 +59,64 @@ class FolderMatch(deskbar.Match.Match):
 class FileFolderHandler(deskbar.Handler.Handler):
 	def __init__(self):
 		deskbar.Handler.Handler.__init__(self, gtk.STOCK_OPEN)
-				
-	def query(self, query, max):
+		self.cache = {}
+		
+	def initialize(self):
+		# Disables the file indexing functionnality as it's too slow
+		pass
+#		def add_files(files):
+#			for f in files:
+#				self.cache[basename(f).lower()] = f
+#			
+#		def add_files_dirs(dir, depth=0):
+#			if depth >= 4:
+#				return
+#			
+#			files = []
+#			for f in os.listdir(dir):
+#				if f.startswith("."):
+#					continue
+#					
+#				f = join(dir, f)
+#				if isdir(f):
+#					add_files_dirs(f, depth+1)
+#				
+#				files.append(f)
+#			
+#			gobject.idle_add(add_files, files)
+#		
+#		Thread (None, add_files_dirs, args=(abspath(expanduser("~")),)).start ()
+
+		
+	def query(self, query):
+		
 		result = []
-		result += self.query_filefolder(query, False)[:max]
-		result += self.query_filefolder(query, True)[:max]
+		result += self.query_filefolder(query, False)
+		result += self.query_filefolder(query, True)
+		
+#		query = query.lower()
+#		for key, absname in self.cache.items():
+#			if len(result) >= 50:
+#				break
+#			
+#			if not exists(absname):
+#				del self.cache[key]
+#				continue
+#			
+#			if key.startswith(query):
+#				if isdir(absname):
+#					result += [FolderMatch(self, basename(absname), absname)]
+#				else:
+#					result += [FileMatch(self, basename(absname), absname)]
+#				
 		return result
 	
 	def query_filefolder(self, query, is_file):
 		completions, prefix, relative = filesystem_possible_completions(query, is_file)
 		if is_file:
-			return [FileMatch(self, join(prefix, basename(completion)), completion) for completion in completions]
+			return [FileMatch(self, join(prefix, basename(completion)), "file://"+completion) for completion in completions]
 		else:
-			return [FolderMatch(self, join(prefix, basename(completion)), completion) for completion in completions]
+			return [FolderMatch(self, join(prefix, basename(completion)), "file://"+completion) for completion in completions]
 			
 def filesystem_possible_completions(prefix, is_file=False):
 	"""
@@ -86,11 +133,14 @@ def filesystem_possible_completions(prefix, is_file=False):
 		relative = True
 		prefix = join("~/", prefix)
 	# Path starting with ~test are considered in ~/test
-	if prefix.startswith("~") and not prefix.startswith("~/"):
+	if prefix.startswith("~") and not prefix.startswith("~/") and len(prefix) > 1:
 		prefix = join("~/", prefix[1:])
 	if prefix.endswith("/"):
 		prefix = prefix[:-1]
 		
+	if prefix == "~":
+		return ([expanduser(prefix)], dirname(expanduser(prefix)), relative)
+
 	# Now we see if the typed name matches exactly a file/directory, or
 	# If we must take the parent directory and match the beginning of each file
 	start = None
@@ -109,11 +159,12 @@ def filesystem_possible_completions(prefix, is_file=False):
 	# First if we have an exact file match, and we requested file matches we return it alone,
 	# else, we return the empty file set
 	if my_isfile(path):
+		print 'Myisfile:', is_file
 		if is_file:
 			return ([path], dirname(prefix), relative)
 		else:
 			return ([], prefix, relative)
-			
+
 	return ([f
 		for f in map(lambda x: join(path, x), os.listdir(path))
 		if my_isfile(f) == is_file and not basename(f).startswith(".") and (start == None or basename(f).startswith(start))
