@@ -53,7 +53,7 @@ class CuemiacCategory :
 	"""
 	A class representing a root node in the cuemiac model/view.
 	"""
-	def __init__ (self, id, parent, threshold=5):
+	def __init__ (self, id, parent):
 		"""
 		name: i18n'ed name for the category
 		parent: CuemiacTreeStore in which this category belongs
@@ -66,12 +66,18 @@ class CuemiacCategory :
 		try:
 			self.__name = CATEGORIES[id]["name"]
 			self.__id = id
+			if "threshold" in CATEGORIES[id]:
+				self.__threshold = CATEGORIES[id]["threshold"]
+			else:
+				#FIXME: this needs to be a really big number..
+				self.__threshold = 100000
 		except:
 			self.__name = CATEGORIES["default"]["name"]
+			self.__threshold = CATEGORIES["default"]["threshold"]
 			self.__id = "default"
+
 			
 		self.__priority = -1
-		self.__threshold = threshold
 		self.__count = 0
 
 	def get_category_row_path (self):
@@ -124,7 +130,7 @@ class CuemiacCategory :
 		return self.__priority
 	
 	def set_priority(self, match):
-		p = match.get_handler().get_priority()
+		p = match.get_priority()[0]
 		if self.__priority < p:
 			self.__priority = p
 			
@@ -190,11 +196,11 @@ class CuemiacModel (gtk.TreeStore):
 		text1, match1 = match1
 		text2, match2 = match2
 		
-		diff = match1.get_handler().get_priority() - match2.get_handler().get_priority()
+		diff = match1.get_priority()[0] - match2.get_priority()[0]
 		if diff != 0:
 			return diff
 			
-		diff = match1.get_priority() - match2.get_priority()
+		diff = match1.get_priority()[1] - match2.get_priority()[1]
 		if diff != 0:
 			return diff
 		
@@ -300,6 +306,9 @@ class CuemiacModel (gtk.TreeStore):
 		
 	def paths_equal (self, path1, path2):
 		"""Returns true if the two paths point to the same cell."""
+		if path1 == None or path2 == None:
+			return False
+			
 		return ( self.get_string_from_iter (self.get_iter(path1)) == self.get_string_from_iter (self.get_iter(path2)) )
 
 class CellRendererCuemiacCategory (gtk.CellRendererText):
@@ -416,8 +425,9 @@ class CuemiacTreeView (gtk.TreeView):
 		
 		self.connect ("cursor-changed", self.__on_cursor_changed)
 		self.set_property ("headers-visible", False)
-		self.set_property ("hover-selection", True)
-		self.connect ("button-press-event", self.__on_click)
+		self.connect ("row-activated", lambda w,p,c: self.__on_activated())
+		self.connect ("button-press-event", lambda w,e: self.__on_activated(self.get_path_at_pos(int(e.x), int(e.y))[0]))
+		
 		self.connect ("key-press-event", self.__on_key_press)
 				
 		self.set_enable_search (False)
@@ -541,11 +551,15 @@ class CuemiacTreeView (gtk.TreeView):
 		
 		cell.set_property ("markup", model[iter][model.ACTIONS])
 
-	def __on_click (self, widget, event):
-		model, iter = self.get_selection().get_selected()
+	def __on_activated (self, path=None):
+		if path == None:
+			model, iter = self.get_selection().get_selected()
+		else:
+			model, iter = self.get_model(), path
 		match = model[iter][model.MATCHES]
 		self.emit ("match-selected", match)
-				
+		
+		
 	def __on_key_press (self, widget, event):
 		# FIXME: In the future we should check for ctrl being pressed to create shortcuts
 		model, iter = self.get_selection().get_selected()
@@ -562,7 +576,7 @@ class CuemiacTreeView (gtk.TreeView):
 					self.expand_row (path, False)
 				return True
 			
-			self.emit ("match-selected", match)
+		return False
 		
 
 class CuemiacUI (DeskbarUI):
@@ -584,7 +598,7 @@ class CuemiacUI (DeskbarUI):
 		self.entry = self.icon_entry.get_entry ()
 		self.entry_icon = gtk.Image ()
 		self.history = get_deskbar_history ()
-		self.history_popup = CuemiacHistoryPopup (self.history, self.deskbar_button.button_arrow, applet)
+		self.history_popup = CuemiacHistoryPopup (self.deskbar_button.button_arrow, applet)
 		self.model = CuemiacModel ()
 		self.cview = CuemiacTreeView (self.model)
 		self.scroll_win = gtk.ScrolledWindow ()
@@ -619,15 +633,15 @@ class CuemiacUI (DeskbarUI):
 		
 		self.screen_height = self.popup.get_screen().get_height ()
 		self.screen_width = self.popup.get_screen().get_width ()
-		self.max_window_height = int (0.6 * self.screen_height)
-		self.max_window_width = int (0.4 * self.screen_width)
+		self.max_window_height = int (0.8 * self.screen_height)
+		self.max_window_width = int (0.6 * self.screen_width)
 
 		self.box.show ()
 		self.icon_entry.show_all ()
 		
 		self.set_sensitive(False)
 		try:
-			self.applet.set_background_widget(self.deskbar_button)
+			self.applet.set_background_widget(applet)
 		except Exception, msg:
 			pass
 		
@@ -635,6 +649,11 @@ class CuemiacUI (DeskbarUI):
 		
 		self.applet.set_applet_flags(gnomeapplet.EXPAND_MINOR)
 		self.applet.set_flags(gtk.CAN_FOCUS)
+	
+	def close_view(self):
+		self.deskbar_button.button_arrow.set_active (False)
+		self.deskbar_button.button_main.set_active (False)
+		self.emit ("stop-query")
 		
 	def update_entry_icon (self, icon=None):
 		if icon == None:
@@ -648,6 +667,7 @@ class CuemiacUI (DeskbarUI):
 					icon=match.get_icon()
 				
 		self.entry_icon.set_property('pixbuf', icon)
+		self.entry_icon.set_size_request(deskbar.ICON_WIDTH, deskbar.ICON_HEIGHT)
 		
 	def on_match_selected (self, cview, match, is_historic=False):
 		if match.__class__ == Nest or match.__class__ == CuemiacCategory:
@@ -660,6 +680,10 @@ class CuemiacUI (DeskbarUI):
 	
 	def show_entry (self, time=None):
 		if self.deskbar_button.get_active_main ():
+			# Unselect what we have in the entry, so we don't occupy the middle-click-clipboard
+			# thus clearing the model on popup
+			self.entry.select_region (0,0)
+		
 			# If the entry is empty or there's something in the middle-click-clipboard
 			# clear the popup so that we can paste into the entry.
 			if self.entry.get_text().strip() == "" or self.clipboard.wait_for_text():
@@ -669,7 +693,7 @@ class CuemiacUI (DeskbarUI):
 				
 			self.deskbar_button.button_arrow.set_active (False)
 			self.adjust_popup_size ()
-			self.popup.update_position ()
+			# self.popup.update_position ()
 			self.update_entry_icon ()
 			
 			if time != None:
@@ -679,9 +703,6 @@ class CuemiacUI (DeskbarUI):
 			
 			self.entry.grab_focus ()
 		else:
-			# Unselect what we have in the entry, so we don't occupy the middle-click-clipboard
-			# thus clearing the model on next popup
-			self.entry.select_region (0,0)
 			self.popup.hide ()
 			self.emit ("stop-query")
 	
@@ -689,12 +710,13 @@ class CuemiacUI (DeskbarUI):
 		# Toggle expandedness of the popup
 		self.deskbar_button.button_main.set_active(not self.deskbar_button.button_main.get_active())
 		# This will focus the entry since we are passing the real event time and not the toggling time
-		self.show_entry(time)
+		if self.deskbar_button.button_main.get_active():
+			self.show_entry(time)
 		
 	def show_history (self):
 		if self.deskbar_button.get_active_arrow ():
 			self.deskbar_button.button_main.set_active (False)
-			self.history_popup.update_position ()
+			# self.history_popup.update_position ()
 			self.history_popup.show_all ()
 		else:
 			self.history_popup.hide ()
@@ -720,7 +742,7 @@ class CuemiacUI (DeskbarUI):
 		
 		if applet.get_size() <= 36:
 			image_name += ".png"
-			s = 24
+			s = -1
 		else:
 			image_name += ".svg"
 			s = applet.get_size()-12
@@ -731,21 +753,19 @@ class CuemiacUI (DeskbarUI):
 		if self.invalid :
 			self.invalid = False
 			self.model.clear()
+			
 		entry_text = self.entry.get_text().strip()
-		valid_matches = False
 		for text, match in matches:
-			if text == entry_text: # FIXME: Maybe it will suffice to only check the first match
-				self.model.append ((text,match))
-				valid_matches = True
-		if valid_matches:
-			self.popup.show_all ()
-	
+			self.model.append ((text, match))
+		self.popup.show_all ()
+		
 	def middle_click(self):
 		text = self.clipboard.wait_for_text ()
 		if text != None:
 			self.deskbar_button.button_main.set_active (True)
 			self.entry.grab_focus()
 			self.entry.set_text(text)
+			return True
 		
 	def set_layout_by_orientation (self, orient, reshow=True, setup=False):
 		"""orient should be a gnomeapplet.ORIENT_{UP,DOWN,LEFT,RIGHT}.
@@ -802,8 +822,7 @@ class CuemiacUI (DeskbarUI):
 			return
 		
 		self.invalid = True
-		self.popup.show ()
-		self.emit ("start-query", qstring, 100)
+		self.emit ("start-query", qstring)
 	
 	def hide_if_entry_empty (self):
 		"""Checks if the entry is empty, and hides the window if so.
